from .__dependencies import *

animations = [[".(^-^)'", "-(^-^)-", "'(^-^).", "-(^-^)-", ".(^-^)'", "-(^-^)-", "'(^-^).", "-(^-^)-"],
              ["▯▯▯▯ 25%", "▯▯▯▯▯▯▯▯ 50%", "▯▯▯▯▯▯▯▯ 70%", "\\"],
              ["", "", "", "", ""]
            ]


def open_url(url):
    webbrowser.open(url)

def path_converter(file_name):
    return pkg_resources.resource_filename('disdial', file_name)

def welcome_message():
    print(colored("Welcome to DisDial!\n", "green"))

def load_animation(pref="Loading... ", i=0):
    load_str = pref
    ls_len = len(load_str)

    animation = animations[i]
    anicount = 0

    counttime = 0
    i = 0
    count = 0

    while count < 10:
        time.sleep(0.1)

        load_str_list = list(load_str)

        res = ''
        for j in range(ls_len):
            res = res + load_str_list[j]

        print("\r" + res + animation[anicount], end='', flush=True)

        load_str = res

        anicount = (anicount + 1) % 4
        i = (i + 1) % ls_len
        counttime = counttime + 1

        count += 1

    os.system('cls' if os.name == 'nt' else 'clear')

def loading_animation2():
    animations = [
                "24.3%  ▯▯▯▯▯▯", 
                "58.7% ▯▯▯▯▯▯▯▯▯▯▯▯", 
                "75.1% ▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯", 
                "99.9% ▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯▯"
                ]

    for animation in animations:
        print(animation, end='', flush=True)
        time.sleep(random.uniform(0.12, 0.25))
        print("\r", end='', flush=True)
        
    os.system('cls' if os.name == 'nt' else 'clear')


import pytz
import requests
import datetime

import json
import random
import threading
import argparse
import threading
import webbrowser
import keyboard
import pkg_resources
import sys, time, os
from termcolor import colored


from .__utils import *
from .__chat import main as __chat_main
from .__chat import update_screen, send_message_to_server, get_all_messages
from .__user_name import main as check_username


def main():
    try:
        parser = argparse.ArgumentParser(description="This is a library designed to enable one to chat from the CLI.")
        parser.add_argument("-d", "--docs", action="store_true", help="documentation about the library.")    
        args = parser.parse_args()
        
        if not any(vars(args).values()):
            check_username()
            
            load_animation()
            welcome_message()
            response = __chat_main()
            loading_animation2()

            for message in get_all_messages():
                print(message)

            while True:
                try:
                    message = input(f"\n>> ")
                    if message=="$r":
                        update_screen()
                        continue
                    send_message_to_server(message)
                    update_screen()
                except EOFError:
                    break

        if args.docs:
            open_url("https://youtu.be/-p0a9BJTEvA")
            
    except KeyboardInterrupt:
        print("\n\nExiting...")
        exit(0)
        
def auto_update_screen(delay=10):
    time.sleep(delay)
    update_screen()

if __name__ == "__main__":
    auto_update_thread = threading.Thread(target=auto_update_screen)
    auto_update_thread.start()
    threading.Thread(target=main).start()


from .__constants import *
from .__user_name import main as __username_main
from .__dependencies import requests, datetime, pytz, json, os

token = json.load(open(JSON_FILE_PATH))['token']
headers = {
    'Authorization': token,
    'Content-Type': 'application/json',
}

def get_time():
    current_time = datetime.datetime.now(pytz.timezone('Asia/Kolkata'))
    return current_time.strftime('%Y-%m-%d %H:%M:%S %Z')

def read_logs():
    with open(LOGS_FILE_PATH, "r") as f:
        return f.read()

def update_logs(message):
    with open(LOGS_FILE_PATH, "w") as f:
        f.write(message)

def get_last_message():
    return get_all_messages()[-1]

def get_all_messages():
    response = requests.get(API_URL2, headers=headers)
    data = response.json()[::-1]
    data = [x for x in data if data != ""]
    return data

def check_new_message():
    last_message = get_last_message()
    return (True, last_message) if last_message != read_logs() else (False, last_message)

def send_message_to_server(message):
    current_time = get_time()
    username = __username_main()
    
    data = {
            'time': current_time,
            'username': username,
            'message': message, 
            }
    
    response = requests.post(API_URL, json=data, headers=headers)
        
    if response.status_code == 200:
        update_logs(f"[{current_time}] {username}: {data['message']}")
        return f"[{current_time}] {username}: {data['message']}"
    else:
        return f"request failed : {response.status_code}"
        
def update_screen():
    os.system('cls' if os.name == 'nt' else 'clear')
    
    for msg in get_all_messages():
        print(msg)


def main():
    if check_new_message()[0]:
        update_logs(get_all_messages()[-1])
        return get_all_messages()
    else: 
        return False


from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="disdial",
    version="0.1.1",
    author="Arpit Sengar (arpy8)",
    description="This library is under developmental stage.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["wheel", "termcolor", "setuptools", "argparse", "requests", "pytz", "argparse", "datetime"],
    entry_points={
        "console_scripts": [
            "disdial=disdial.main:main",
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    packages=find_packages(),
    package_data={'my_package': ['data/*.txt', 'data/*.json']},
)


from .__utils import *
from .__constants import *

def get_username():
    try:
        with open(JSON_FILE_PATH, "r") as f:
            return json.load(f)["name"]
        
    except Exception as e:
        print(colored(f"Error setting username: {e}", "red"))

def set_username(username):
    try:
        with open(JSON_FILE_PATH, 'r') as f:
            data = json.load(f)
            
        data['name'] = username
        
        with open(JSON_FILE_PATH, 'w') as f:
            json.dump(data, f, indent=2)
            
        print(colored(f"username set to {username}", "green"))

    except Exception as e:
        print(colored(f"error setting username: {e}", "red"))

def main():
    name = get_username()
    
    if get_username() == "NULL":
        name = input("Please enter your name: ")
        set_username(name)
    
    return name


from .__utils import path_converter

API_URL = f'https://disdial.onrender.com/data'
API_URL2 = f'https://disdial.onrender.com/all'
# API_URL = f'https://127.0.0.1:5000/data'

LOGS_FILE_PATH = path_converter("data/logs.txt")
JSON_FILE_PATH = path_converter("data/config.json")